@@ -58,7 +58,7 @@
 const uint16_t MAX_VOICES = 32;			// polyphony
 const uint8_t  MAX_NOTES  = 128;
 
-const float MIN_ENV_MSECS = 2.0f;		// min 2msec per stage
+const float MIN_ENV_MSECS = 2.0f;		// min 2 msec per stage
 const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage (default)
 
 const float DETUNE_SCALE  = 0.5f;
@@ -341,6 +341,28 @@ struct samplv1_env
 		p->c0 = p->value;
 	}
 
+	void restart(State *p, bool legato)
+	{
+		p->running = true;
+		if (legato) {
+			p->stage = Decay;
+			p->frames = min_frames;
+			p->phase = 0.0f;
+			p->delta = 1.0f / float(p->frames);
+			p->c1 = *sustain - p->value;
+			p->c0 = 0.0f;
+		} else {
+			p->stage = Attack;
+			p->frames = uint32_t(*attack * *attack * max_frames);
+			p->phase = 0.0f;
+			if (p->frames < min_frames)
+				p->frames = min_frames;
+			p->delta = 1.0f / float(p->frames);
+			p->c1 = 1.0f;
+			p->c0 = 0.0f;
+		}
+	}
+
 	// parameters
 
 	samplv1_port attack;
@@ -1084,7 +1106,7 @@ void samplv1_impl::updateEnvTimes (void)
 	if (envtime_msecs < MIN_ENV_MSECS)
 		envtime_msecs = (gen1_sample.length() >> 1) / srate_ms;
 	if (envtime_msecs < MIN_ENV_MSECS)
-		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+		envtime_msecs = MIN_ENV_MSECS * 2.0f;
 
 	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
 	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
@@ -1309,14 +1331,17 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			samplv1_voice *pv;
 			// mono voice modes
 			if (*m_def.mono > 0.0f) {
+				int n = 0;
 				for (pv = m_play_list.next(); pv; pv = pv->next()) {
 					if (pv->note >= 0
 						&& pv->dca1_env.stage != samplv1_env::Release) {
 						m_dcf1.env.note_off_fast(&pv->dcf1_env);
 						m_lfo1.env.note_off_fast(&pv->lfo1_env);
 						m_dca1.env.note_off_fast(&pv->dca1_env);
-						m_notes[pv->note] = NULL;
-						pv->note = -1;
+						if (++n > 1) { // there shall be only one
+							m_notes[pv->note] = NULL;
+							pv->note = -1;
+						}
 					}
 				}
 			}
@@ -1394,6 +1419,21 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					m_lfo1.env.note_off(&pv->lfo1_env);
 					pv->gen1.setLoop(false);
 				}
+				m_notes[pv->note] = NULL;
+				pv->note = -1;
+				// mono legato?
+				if (*m_def.mono > 0.0f) {
+					do pv = pv->prev();	while (pv && pv->note < 0);
+					if (pv && pv->note >= 0) {
+						const bool legato = (*m_def.mono > 1.0f);
+						m_dcf1.env.restart(&pv->dcf1_env, legato);
+						m_lfo1.env.restart(&pv->lfo1_env, legato);
+						m_dca1.env.restart(&pv->dca1_env, legato);
+						pv->gen1.setLoop(*m_gen1.loop > 0.0f);
+						if (!legato) pv->gen1.start();
+						m_notes[pv->note] = pv;
+					}
+				}
 			}
 		}
 		// key pressure/poly.aftertouch
@@ -1526,6 +1566,8 @@ void samplv1_impl::allSustainOff (void)
 				m_dcf1.env.note_off(&pv->dcf1_env);
 				m_lfo1.env.note_off(&pv->lfo1_env);
 				pv->gen1.setLoop(false);
+				m_notes[pv->note] = NULL;
+				pv->note = -1;
 			}
 		}
 		pv = pv->next();
@@ -1799,9 +1841,8 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				m_dca1.env.next(&pv->dca1_env);
 
 			if (pv->dca1_env.stage == samplv1_env::Idle || pv->gen1.isOver()) {
-				if (pv->note >= 0)
-					m_notes[pv->note] = NULL;
-				free_voice(pv);
+				if (pv->note < 0)
+					free_voice(pv);
 				nblock = 0;
 			} else {
 				if (pv->dcf1_env.running && pv->dcf1_env.frames == 0)
