@@ -31,6 +31,8 @@
 #include "samplv1_controls.h"
 #include "samplv1_programs.h"
 
+#include "ui_samplv1widget.h"
+
 #include <QMessageBox>
 #include <QDir>
 #include <QTimer>
@@ -45,7 +47,7 @@
 
 // Constructor.
 samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
-	: QWidget(pParent, wflags)
+	: QWidget(pParent, wflags), p_ui(new Ui::samplv1widget), m_ui(*p_ui)
 {
 	Q_INIT_RESOURCE(samplv1);
 
@@ -214,12 +216,15 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		channels << QString::number(iChannel + 1);
 
 	m_ui.Def1ChannelKnob->insertItems(0, channels);
-#if 0
-	// Mono switches
-	m_ui.Def1MonoKnob->insertItems(0, states);
-#else
-	m_ui.Def1MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
-#endif
+
+	// Mono switches.
+	QStringList modes;
+	modes << tr("Poly");
+	modes << tr("Mono");
+	modes << tr("Legato");
+
+	m_ui.Def1MonoKnob->insertItems(0, modes);
+
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -533,6 +538,8 @@ samplv1widget::~samplv1widget (void)
 {
 	if (m_sched_notifier)
 		delete m_sched_notifier;
+
+	delete p_ui;
 }
 
 
@@ -877,7 +884,7 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		samplv1_param::loadPreset(pSamplUi->instance(), sFilename);
+		pSamplUi->loadPreset(sFilename);
 
 	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
 }
@@ -891,7 +898,7 @@ void samplv1widget::savePreset ( const QString& sFilename )
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		samplv1_param::savePreset(pSamplUi->instance(), sFilename);
+		pSamplUi->savePreset(sFilename);
 
 	const QString& sPreset
 		= QFileInfo(sFilename).completeBaseName();
@@ -977,6 +984,11 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
 		activateParamKnobs(pSample->filename() != NULL);
 		updateSampleLoop(pSample);
+		// Set current preset name if empty...
+		if (pSample->filename() && m_ui.Preset->preset().isEmpty()) {
+			m_ui.Preset->setPreset(
+				QFileInfo(pSample->filename()).completeBaseName());
+		}
 	} else {
 		m_ui.Gen1Sample->setLoop(false);
 		m_ui.Gen1Sample->setLoopStart(0);
