@@ -122,7 +122,7 @@ void samplv1widget_sample::setSample ( samplv1_sample *pSample )
 		const int w = width() & 0x7ffe; // force even.
 		const int w2 = (w >> 1);
 		const unsigned int nframes = m_pSample->length();
-		const unsigned int nperiod = nframes / w2;
+		const float period = float(nframes) / float(w2);
 		const int h0 = h / m_iChannels;
 		const float h1 = float(h0 >> 1);
 		int y0 = h1;
@@ -134,18 +134,18 @@ void samplv1widget_sample::setSample ( samplv1_sample *pSample )
 			float vmin = 0.0f;
 			int n = 0;
 			int x = 1;
-			unsigned int j = nperiod;
+			unsigned int j = 0;
 			for (unsigned int i = 0; i < nframes; ++i) {
 				const float v = *pframes++;
 				if (vmax < v)
 					vmax = v;
 				if (vmin > v)
 					vmin = v;
-				if (++j > nperiod) {
+				if (i > j) {
 					m_ppPolyg[k]->setPoint(n, x, y0 - int(vmax * h1));
 					m_ppPolyg[k]->setPoint(w - n - 1, x, y0 - int(vmin * h1));
-					j = 0; vmax = vmin = 0.0f;
-					++n; x += 2;
+					vmax = vmin = 0.0f;
+					++n; x += 2; j = (unsigned int) (float(n) * period);
 				}
 			}
 			while (n < w2) {
@@ -508,9 +508,20 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 
     painter.fillRect(rect, rgbDark);
 
-	if (!m_sName.isEmpty()) {
+	QString sTitle = m_sName;
+	if (m_pSample && m_pSample->filename()) {
+		if (!sTitle.isEmpty()) {
+			sTitle += ' ';
+			sTitle += '-';
+			sTitle += ' ';
+		}
+		sTitle += QFileInfo(
+			QString::fromUtf8(m_pSample->filename())
+		).completeBaseName();
+	}
+	if (!sTitle.isEmpty()) {
 		painter.setPen(pal.midlight().color());
-		painter.drawText(rect.adjusted(2, 0, -2, -0), Qt::AlignLeft, m_sName);
+		painter.drawText(rect.adjusted(2, 0, -2, -0), Qt::AlignLeft, sTitle);
 	}
 
 	if (m_pSample && m_ppPolyg) {
@@ -564,7 +575,9 @@ void samplv1widget_sample::openSample (void)
 	if (pConfig == NULL)
 		return;
 
-	QString sFilename;
+	QString sFilename = pConfig->sSampleDir;
+	if (m_pSample && m_pSample->filename())
+		sFilename = QString::fromUtf8(m_pSample->filename());
 
 	// Cache supported file-types stuff from libsndfile...
 	static QStringList s_filters;
@@ -598,22 +611,23 @@ void samplv1widget_sample::openSample (void)
 
 	const QString& sTitle  = tr("Open Sample") + " - " SAMPLV1_TITLE;
 	const QString& sFilter = s_filters.join(";;");
-#if 1//QT_VERSION < 0x040400
+	QWidget *pParentWidget = NULL;
 	QFileDialog::Options options = 0;
-	if (pConfig->bDontUseNativeDialogs)
+	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
-	sFilename = QFileDialog::getOpenFileName(parentWidget(),
-		sTitle, pConfig->sSampleDir, sFilter, NULL, options);
+		pParentWidget = parentWidget();
+	}
+#if 1//QT_VERSION < 0x040400
+	sFilename = QFileDialog::getOpenFileName(pParentWidget,
+		sTitle, sFilename, sFilter, NULL, options);
 #else
-	QFileDialog fileDialog(nativeParentWidget(),
-		sTitle, pConfig->sSampleDir, sFilter);
+	QFileDialog fileDialog(pParentWidget, sTitle, sFilename, sFilter);
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
 	fileDialog.setFileMode(QFileDialog::ExistingFile);
 	QList<QUrl> urls(fileDialog.sidebarUrls());
 	urls.append(QUrl::fromLocalFile(pConfig->sSampleDir));
 	fileDialog.setSidebarUrls(urls);
-	if (pConfig->bDontUseNativeDialogs)
-		fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
+	fileDialog.setOptions(options);
 	if (fileDialog.exec())
 		sFilename = fileDialog.selectedFiles().first();
 #endif
